"""
Linear and semidefinite programming solver.
"""

# This file is part of CVXOPT version 0.8.2.
# Copyright 2004-2007 J. Dahl and L. Vandenberghe.

import math
from cvxopt import base, blas, lapack, cholmod, misc
from cvxopt.base import matrix, spmatrix
cholmod.options['supernodal'] = 2

__all__ = []
options = {}

def conelp(c, kktsolver, Gl=None, hl=None, Gs=None, hs=None, A=None, 
    b=None, primalstart=None, dualstart=None, xnewcopy=matrix, 
    xdot=blas.dot, xaxpy=blas.axpy, xscal=blas.scal, ynewcopy=matrix, 
    ydot=blas.dot, yaxpy=blas.axpy, yscal=blas.scal):
    """

    Solves a pair of primal and dual cone programs

        minimize    <c,x>             
        subject to  Gl(x) + sl = hl      
                    Gs(x) + ss = hs
                    A(x) = b                      
                    sl >= 0,  ss >= 0

        maximize    -<hl,zl> - <hs,zs> - <b,y>
        subject to  Gl'(zl) + Gs'(zs) + A'(y) + c = 0
                    zl >= 0,  zs >= 0

    Gl: V -> reals^ml, Gs: V -> S^m1 x ... x S^mN, and A: V -> W are 
    linear mappings, where V and W are real vector spaces.

    Input arguments:

        c is a matrix, list of matrices, dictionary of matrices, ...
        representing a vector in the vector space V.

        Gl is a function Gl(x, y, alpha=1.0, beta=0.0, trans='N'):

            y := alpha*Gl(x) + beta*y if trans is 'N' 
            y := alpha*Gl'(x) + beta*y if trans is 'T'.

        If trans is 'N', x is in V and y is in R^ml.
        If trans is 'T', x is in R^ml and y is in V.

        hl is a dense 'd' matrix of size (ml,1).

        Gs is a function Gs(x, y, alpha=1.0, beta=0.0, trans='N'):

            y := alpha*Gs(x) + beta*y if trans is 'N'
            y := alpha*Gs'(x) + beta*y if trans is 'T'.

        If trans is 'N', x is in V and y is in S^m1 x ... x S^mN.
        If trans is 'T', x is in S^m1 x ... x S^mN and y is in V.

        hs is a list of N square dense 'd' matrices, representing a 
        symmetric block diagonal matrix in 'L' storage.

        A is a function A(x, y, alpha, beta, trans='N'):

            y := alpha*A(x) + beta*y if trans is 'N'
            y := alpha*A'(x) + beta*y if trans is 'T'.

        If trans is 'N', x is in V and y is in W
        If trans is 'T', x is in S^m1 x ... x S^mN and y is in V.

        b is a matrix, list of matrices, dictionary of matrices, ...
        representing a vector in the vector space W.
        
        kktsolver is a function that returns a function for solving
        KKT systems 
        
            [ 0   A'  Gl'          Gs'           ] [ x  ]   [ bx  ]
            [ A   0   0            0             ] [ y  ] = [ by  ].
            [ Gl  0   -diag(d)^2   0             ] [ zl ]   [ bzl ]
            [ Gs  0   0            -r*r'*()*r*r' ] [ zs ]   [ bzs ]
        
        Called as f = kktsolver(di, rti) with di = 1.0./d (a dense 'd' 
        matrix of size (ml,1)) and rti = (r')^{-1} (a list of square 
        dense 'd' matrices of size (mk,mk)).  
        The KKT system is solved by f(x,y,zl,zs).  On entry, x, y, zl, 
        zs contain the righthand side.  On exit, they contain the 
        solution, with zl, zs scaled: d.*zl and r'*zs*r are returned 
        instead of zl and zs.

        xnewcopy, xdot, xscal, xaxpy, ynewcopy, ydot, yscal, yaxpy are 
        Python functions.
        xnewcopy(x), ynewcopy(x) create new copies of vectors in V and 
        W, respectively. 
        xdot(x,y), ydot(x,y) return the inner product of vectors in V 
        and W, respectively.
        xscal(alpha, x), yscal(alpha,x) compute x := alpha*x for x in V
        and W, respectively.
        xaxpy(x, y, alpha=1, beta=0), yaxpy(x, y, alpha=1, beta=0) 
        compute y := alpha*x + y for vectors in V and W, respectively.

    conelp() returns a dictionary with keys 'status', 'x', 'sl', 
    'ss', 'y', 'zl', 'zs'.

    The control parameters can be modified by adding an entry to the 
    dictionary options.  

        options['show_progress'] True/False (default: True)
        options['maxiters'] positive integer (default: 100)
        options['abstol'] scalar (default: 1e-7)
        options['reltol'] scalar (default: 1e-7)
        options['feastol'] scalar (default: 1e-7)
        options['refinement'] True/False (default: True)
    """

    EXPON = 3
    STEP = 0.99

    try: DEBUG = options['debug']
    except KeyError: DEBUG = False

    try: MAXITERS = options['maxiters']
    except KeyError: MAXITERS = 100
    else:
        if type(MAXITERS) is not int or MAXITERS < 1:
           raise ValueError, "options['maxiters'] must be a positive "\
               "integer"

    try: ABSTOL = options['abstol']
    except KeyError: ABSTOL = 1e-7
    else:
        if type(ABSTOL) is not float and type(ABSTOL) is not int:
            raise ValueError, "options['abstol'] must be a scalar"

    try: RELTOL = options['reltol']
    except KeyError: RELTOL = 1e-7
    else:
        if type(RELTOL) is not float and type(RELTOL) is not int:
            raise ValueError, "options['reltol'] must be a scalar"

    try: FEASTOL = options['feastol']
    except KeyError: FEASTOL = 1e-7
    else:
        if type(FEASTOL) is not float and type(FEASTOL) is not int:
            raise ValueError, "options['feastol'] must be a scalar"

    try: show_progress = options['show_progress']
    except KeyError: show_progress = True

    try: refinement = options['refinement']
    except KeyError: refinement = True

    def f0(x, y, alpha=1.0, beta=0.0, trans='N'):  
        # y := alpha*F(x)  + beta*y  (trans = 'N') 
        # y := alpha*F'(x) + beta*y  (trans = 'T')
        # when F : R^n -> a 0-dimensional space
        if trans == 'N':  pass
        else: xscal(beta, y)
    if Gl is None: Gl = f0
    if Gs is None: Gs = f0
    if A is None: A = f0
    if hl is None: hl = matrix(0.0, (0,1))
    if hs is None: hs = []
    if b is None: b = matrix(0.0, (0,1))
    ml, ms, p = hl.size[0], [ h_k.size[0] for h_k in hs ], b.size[0]
    maxm = max( [0] + ms )
    m = ml + sum(ms) 

    def xcopy(x, y): xscal(0, y); xaxpy(x, y)
    def ycopy(x, y): yscal(0, y); yaxpy(x, y)

    x = xnewcopy(c);  xscal(0,x)
    sl = matrix(1.0, (ml,1))
    ss = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    for sk in ss:  sk[::sk.size[0]+1] = 1.0
    y = ynewcopy(b);  yscal(0,y)
    zl = matrix(1.0, (ml,1))
    zs = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    for zk in zs:  zk[::zk.size[0]+1] = 1.0

    resx, hresx = xnewcopy(c), xnewcopy(c)
    reszl, hreszl = matrix(0.0, (ml,1)), matrix(0.0, (ml,1))
    reszs = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    hreszs = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    resy, hresy = ynewcopy(b), ynewcopy(b)
    dx = xnewcopy(c)
    dsl = matrix(0.0, (ml,1))
    dss = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    dy = ynewcopy(b)
    dzl = matrix(0.0, (ml,1))
    dzs = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    dkappa, dtau = matrix(0.0, (1,1)), matrix(0.0, (1,1))
    dx1 = xnewcopy(c)
    dy1 = ynewcopy(b)
    dsl1 = matrix(0.0, (ml,1))
    dss1 = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    dzl1 = matrix(0.0, (ml,1))
    dzs1 = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    if refinement:
        dx2 = xnewcopy(c)
        dy2 = ynewcopy(b)
        dsl2 = matrix(0.0, (ml,1))
        dss2 = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
        dzl2 = matrix(0.0, (ml,1))
        dzs2 = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
        dkappa2, dtau2 = matrix(0.0, (1,1)), matrix(0.0, (1,1))
    r = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    rti = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    lmbda = matrix(0.0, (m+1,1))
    gamma = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
    thl = matrix(0.0, (ml,1))
    ths = [ matrix(0.0, (m_k, m_k)) for m_k in ms ]
    sigs, sigz = matrix(0.0, (m+1,1)), matrix(0.0, (m+1,1))
    ones = matrix(1.0, (max(maxm,ml),1))
    work, work2 = matrix(0.0, (maxm,maxm)), matrix(0.0, (maxm,maxm))


    # Initial points more or less as in Mehrotra's paper.

    if primalstart is None or dualstart is None:

        # Factor
        #
        #     [ 0   A'  Gl' Gs' ] 
        #     [ A   0   0   0   ]
        #     [ Gl  0  -I   0   ]
        #     [ Gs  0   0  -I   ]
        
        dli = matrix(1.0, (ml,1)) 
        for e in rti:
            blas.scal(0.0, e)
            e[::e.size[0]+1] = 1.0
        try: 
            f = kktsolver(dli, rti)
        except ArithmeticError:  
            raise ValueError, "Rank(A) < p or Rank([Gl; Gs; A]) < n"

    if primalstart is None:

        # minimize    || Gl(x) - hl ||^2
        # subject to  A(x) = b
        #
        # by solving
        #
        # [ 0   A'  Gl' Gs' ] [x  ]   [0 ]
        # [ A   0   0   0   ] [dy ] = [b ]
        # [ Gl  0  -I   0   ] [dzl]   [hl]
        # [ Gs  0   0  -I   ] [dzs]   [hs]
        #
        # (sl, ss) = (hl-Gl*x, hs-Gs(x)) and add a multiple of the
        # identity to make (sl,ss) >= 0. 

        xscal(0.0, x)
        ycopy(b, dy)  
        blas.copy(hl, dzl)
        misc.scopy(hs, dzs)
        f(x, dy, dzl, dzs) 
        blas.scal(0.0, sl);  blas.axpy(dzl, sl, alpha=-1)
        misc.sscal(0.0, ss);  misc.saxpy(dzs, ss, alpha=-1)
        blas.copy(sl, lmbda)
        if sl: ap = min(sl)
        else: ap = 0.0
        for e in ss:
            if e.size[0]:
                lapack.syevr(+e, lmbda, range='I', il=1, iu=1)
                ap = min(ap, lmbda[0])
        if ap < 0.0:  
            sl += -1.5*ap 
            for e in ss: e[::e.size[0]+1] -= 1.5*ap 

    else:
        xcopy(primalstart['x'], x)
        if ml: blas.copy(primalstart['sl'], sl)
        if ms: misc.scopy(primalstart['ss'], ss)


    if dualstart is None:

        # minimize   || zl ||^2
        # subject to Gl'(zl) + A'(y) + c = 0
        #
        # by solving
        #
        # [ 0   A'  Gl'  Gs' ] [dx]   [-c]
        # [ A   0   0    0   ] [y ] = [ 0]
        # [ Gl  0  -I    0   ] [zl]   [ 0]
        # [ Gs  0   0   -I   ] [zs]   [ 0]

        xcopy(c, dx); xscal(-1.0, dx)
        yscal(0.0, y)
        blas.scal(0.0, zl)
        misc.sscal(0.0, zs)
        f(dx, y, zl, zs)
        blas.copy(zl, lmbda)
        if zl: ad = min(zl) 
        else: ad = 0.0 
        for e in zs:
            if e.size[0]:
                lapack.syevr(+e, lmbda, range='I', il=1, iu=1)
                ad = min(ad, lmbda[0]) 
        if ad <= 0.0:  
            zl += -1.5*ad 
            for e in zs: e[::e.size[0]+1] -= 1.5*ad 

    else:
        if p: ycopy(dualstart['y'], y)
        if ml: blas.copy(dualstart['zl'], zl)
        if ms: misc.scopy(dualstart['zs'], zs) 

    gap = blas.dot(sl,zl) + misc.sdot(ss, zs)

    if primalstart is None and dualstart is None and gap < 1e-10:
        # If gap is zero, z, s must have zero components, and this means
        # that ad and ap were zero.  Therefore the initial point 
        # happens to be feasible and optimal.  So it does not matter 
        # if (sl,ss) or (zl,zs) are not strictly positive and we can 
        # skip to the computation of the residuals.
        pass

    else: 
        if primalstart is None:  
            # (sl, ss) >= could be non-strictly positive so we need to
            # make it positive definite.  Note that sum(zl) + tr(zs)
            # cannot be zero, because (zl,zs)=0 means dualstart is None
            # and gap = 0, and that was handled above.
            trz = blas.asum(zl) + sum([ blas.asum(e, inc=e.size[0]+1) 
                for e in zs ])                
            a = max(gap, 1e-8) / trz
            sl += a 
            for e in ss: e[::e.size[0]+1] += a

        if dualstart is None:
            # Idem for (zl, zs).
            trs = blas.asum(sl) + sum([ blas.asum(e, inc=e.size[0]+1) 
                for e in ss ])
            a = max(gap, 1e-8) / trs
            zl += a
            for e in zs: e[::e.size[0]+1] += a

    tau, kappa = 1.0, 1.0
    gap = blas.dot(sl, zl) + misc.sdot(ss, zs) 

    for iters in xrange(MAXITERS):

        # hresx = -A'(y) - Gl'(zl) - Gs'(zs)  
        A(y, hresx, alpha=-1.0, trans='T') 
        Gl(zl, hresx, alpha=-1.0, beta=1.0, trans='T') 
        Gs(zs, hresx, alpha=-1.0, beta=1.0, trans='T') 

        # resx = hresx - c*tau = -A'(y) - Gl'(zl) - Gs'(zs) - c*tau
        xcopy(hresx, resx)
        xaxpy(c, resx, alpha=-tau)

        # hresy = A(x)  
        A(x, hresy)

        # resy = hresy - b*tau = A(x) - b*tau
        ycopy(hresy, resy)
        yaxpy(b, resy, alpha=-tau)

        # hreszl = sl + Gl(x)  
        Gl(x, hreszl)
        blas.axpy(sl, hreszl)

        # reszl = hreszl - hl*tau = sl + Gl(x) - hl*tau
        blas.scal(0, reszl)
        blas.axpy(hreszl, reszl)
        blas.axpy(hl, reszl, alpha=-tau)

        # hreszs = ss + Gs(x) 
        Gs(x, hreszs)
        misc.saxpy(ss, hreszs)

        # reszs = hreszs - hs*tau = ss + Gs(x) - hs*tau 
        misc.sscal(0, reszs)
        misc.saxpy(hreszs, reszs)
        misc.saxpy(hs, reszs, alpha=-tau)

        # rest = kappa + <c,x> + <b,y> + <hl,zl> + <hs,zs>
        cx, by = xdot(c,x), ydot(b,y) 
        hlzl, hszs = blas.dot(hl,zl), misc.sdot(hs,zs)
        rest = kappa + cx + by + hlzl + hszs

        # stopping criteria
        pcost, dcost = cx/tau, -(by + hlzl + hszs) / tau        
        nrmhresx = math.sqrt(xdot(hresx, hresx)) 
        nrmresx = math.sqrt(xdot(resx, resx)) / tau
        nrmhresy = math.sqrt(ydot(hresy, hresy))
        nrmresy = math.sqrt(ydot(resy, resy)) / tau
        nrmhreszl = blas.nrm2(hreszl) 
        nrmreszl = blas.nrm2(reszl) / tau 
        nrmhreszs = misc.snrm2(hreszs) 
        nrmreszs = misc.snrm2(reszs) / tau
        if pcost < 0.0:
            relgap = gap / -pcost
        elif dcost > 0.0:
            relgap = gap / dcost
        else: 
            relgap = None

        if iters == 0: 
            nrmresx0 = max(1.0, math.sqrt(xdot(c,c)))
            nrmresy0 = max(1.0, math.sqrt(ydot(b,b)))
            nrmreszl0 = max(1.0, blas.nrm2(hl))
            nrmreszs0 = max(1.0, misc.snrm2(hs))

        if show_progress:
            if iters==0:
                print "% 10s% 12s% 10s% 8s% 7s % 5s" %("pcost", "dcost",
                    "gap", "pres", "dres", "k/t")
            print "%2d: % 8.4e % 8.4e % 4.0e% 7.0e% 7.0e% 7.0e" \
                %(iters, pcost, dcost, gap, max(nrmreszl/nrmreszl0,
                nrmreszs/nrmreszs0, nrmresy/nrmresy0), nrmresx/nrmresx0,
                kappa/tau)

        if max(nrmreszl/nrmreszl0, nrmreszs/nrmreszs0, 
            nrmresy/nrmresy0) <= FEASTOL and \
            nrmresx/nrmresx0 <= FEASTOL and (gap <= ABSTOL or 
            (relgap is not None and relgap <= RELTOL)):
            xscal(1.0/tau, x)
            yscal(1.0/tau, y)
            blas.scal(1.0/tau, sl)
            misc.sscal(1.0/tau, ss)
            blas.scal(1.0/tau, zl)
            misc.sscal(1.0/tau, zs)
            return {'status': 'optimal', 'x': x, 'y': y, 'sl': sl, 
                'ss': ss, 'zl': zl, 'zs': zs}

        elif hlzl + hszs + by < 0.0 and nrmhresx / (-hlzl - hszs - by) \
            <= FEASTOL:
            yscal(1.0/(-hlzl - hszs - by), y)
            blas.scal(1.0/(-hlzl - hszs - by), zl)
            misc.sscal(1.0/(-hlzl - hszs - by), zs)
            return {'status': 'primal infeasible', 'x': None, 
                'sl': None, 'ss': None, 'y': y, 'zl': zl, 'zs': zs}

        elif cx < 0.0 and max(nrmhresy, nrmhreszl, nrmhreszs) / (-cx) \
            <= FEASTOL:
            xscal(1.0/(-cx), x)
            blas.scal(1.0/(-cx), sl)
            misc.sscal(1.0/(-cx), ss)
            return {'status': 'dual infeasible', 'x': x, 'sl': sl, 
                'ss': ss, 'y': None, 'zl': None, 'zs': None}


        if iters == 0:

            # Compute initial scaling.
            #
            # For the linear inequalities: 
            #
            #     dl = sqrt(sl./zl), dli = sqrt(zl./sl)
            #     lmbdl = sqrt(sl.*zl).
            #
            # Store lmbdl in lmbda[:ml].
            #
            # Matrix inequalities: lists of square matrices r, rti with
            #
            #     r[k]'*ss[k]^{-1}*r[k] = diag(lmbds[k])^{-1}
            #     r[k]'*zs[k]*r[k] = diag(lmbds[k])
            #
            # rti[k] is the inverse of r[k]', so that
            #
            #     rti[k]'*ss[k]*rti[k] = diag(lmbds[k])^{-1}
            #     rti[k]'*zs[k]^{-1}*rti[k] = diag(lmbds[k]).
            #
            # Store (lmbds[0], ..., lmbds[N-1]) in lmbda[ml:ml+sum(ms)].
            #
            # Also compute gamma: gamma[k] has entries 
            #
            #     gamma[k][i,j] = 0.5*(lmbds[k][i]+lmbds[k][j])
            #
            # For kappa, tau:  
            #
            #     dg = sqrt(kappa/tau), dgi = sqrt(tau/kappa)
            #     lmbdg = sqrt(tau*kappa)  
            # 
            # Store lmbdg in lmbda[-1].
	
            dli = base.sqrt(misc.ihad(zl, sl)) 
            lmbda[:ml] = base.sqrt(misc.had(sl,zl)) 

            dgi = math.sqrt(tau/kappa)
            lmbda[-1] = math.sqrt(tau*kappa)

            ind = ml
            for k in xrange(len(ms)):

                # Factor ss = L1*L1', zs = L2*L2'.  Store L1 and L2 in 
                # dss and dzs.
                blas.copy(ss[k], dss[k]);  lapack.potrf(dss[k])
                blas.copy(zs[k], dzs[k]);  lapack.potrf(dzs[k])
	 
                # SVD of L2'*L1 = U*diag(lambda)*V'.  Keep U in work. 
                for i in xrange(ms[k]): dss[k][i,i+1:] = 0
                blas.copy(dss[k], work)
                blas.trmm(dzs[k], work, transA='T', ldB=ms[k], n=ms[k],
                    m=ms[k])
                lapack.gesvd(work, lmbda, jobu='O', ldA=ms[k], m=ms[k], 
                    n=ms[k], offsetS=ind)
	       
                # r = L2^{-T}*U*diag(sqrt(lambda)) 
                #   = L1*V*diag(1./sqrt(lambda))
                # rti = L2*U*diag(1./sqrt(lambda)) 
                #     = L1^{-T}*V*diag(sqrt(lambda))
                blas.copy(work, r[k], n=ms[k]*ms[k])
                blas.trsm(dzs[k], r[k], transA='T')
                blas.copy(work, rti[k], n=ms[k]*ms[k])
                blas.trmm(dzs[k], rti[k])
                for i in xrange(ms[k]):
                    a = math.sqrt(lmbda[ind+i])
                    blas.scal(a, r[k], offset=ms[k]*i, n=ms[k])
                    blas.scal(1/a, rti[k], offset=ms[k]*i, n=ms[k])

                # gamma[k][i,j] = .5 * (lambda_i + lambda_j)
                # Use blas.ger to make sure there are no zeros or nan's
                # in upper triangular part.
                blas.scal(0.0, gamma[k])
                blas.ger(lmbda, ones, gamma[k], alpha=.5, offsetx=ind)
                blas.ger(ones, lmbda, gamma[k], alpha=.5, offsety=ind)

                ind += ms[k]

        
        # Define a function 
        #
        #    solve_newton(dx, dy, dzl, dzs, dtau, dsl, dss, dkappa)
        #
        # for solving 
        #
        #     [0     ]   [ 0    A'  Gl'  Gs' 0 ] [dx  ]    [rhsx  ]
        #     [0     ]   [-A    0   0    0   b ] [dy  ]    [rhsy  ]
        #     [dsl   ] - [-Gl   0   0    0   hl] [dzl ] = -[rhszl ]
        #     [dsk   ]   [-Gs   0   0    0   hs] [dzs ]    [rhszs ]
        #     [dkappa]   [-c'  -b' -hl' -hs' 0 ] [dtau]    [rhstau]
        #    
        #     sl.*dzl + zl.*dsl = -rhssl
        #     Hc(dzs*ss + zs*dss) = -rhsss
        #     kappa*dtau + tau*dkappa = -rhskappa.
        #
        # where Hc(x) = 0.5 * (r'*x*(r')^{-1} + r^{-1}*x'*r).
    
        try: f = kktsolver(dli, rti)
        except ArithmeticError:
            if iters == 0 and primalstart and dualstart: 
                raise ValueError, "Rank(A) < p or Rank([Gl; Gs; A]) < n"
            else:
                raise ArithmeticError, "singular KKT matrix"

        # thl := dli .* hl;  ths := rti'*hs*rti
        blas.copy(hl, thl)
        misc.had2(thl, dli)
        misc.cngrnc(rti, hs, ths, trans='T')

        # Solve
        #
        #     [ 0   A'  Gl'         Gs'         ] [dx1 ]          [c ]
        #     [-A   0   0           0           ] [dy1 ] = -dgi * [b ]
        #     [-Gl  0   diag(dl)^2  0           ] [dzl1]          [hl]
        #     [-Gs  0   0           r*r'*()*r*r'] [dzs1]          [hs] 
         
        xcopy(c, dx1);  xscal(-1, dx1)
        ycopy(b, dy1)
        blas.copy(hl, dzl1)
        misc.scopy(hs, dzs1)
        f(dx1, dy1, dzl1, dzs1)
        xscal(dgi, dx1)
        yscal(dgi, dy1)
        blas.scal(dgi, dzl1)
        misc.sscal(dgi, dzs1)
        
        if DEBUG: 
            print "first 4x4 system"

            # unscaled variables
            duzl1 = misc.had(dzl1, dli)
            duzs1 = [ +z_k for z_k in dzs1 ]
            misc.cngrnc(rti, duzs1, duzs1)

            rxdb = xnewcopy(c)
            A(dy1, rxdb, alpha=1, beta=dgi, trans='T')
            Gl(duzl1, rxdb, alpha=1, beta=1, trans='T')
            Gs(duzs1, rxdb, alpha=1, beta=1, trans='T')
            print "    norm(rxdb) = %e" %math.sqrt(xdot(rxdb,rxdb))

            rydb = ynewcopy(b)
            A(dx1, rydb, alpha=-1.0, beta=dgi)
            print "    norm(rydb) = %e" %math.sqrt(ydot(rydb,rydb))

            rzldb = +hl
            Gl(dx1, rzldb, alpha=-1.0, beta=dgi)
            rzldb += misc.ihad(duzl1, misc.had(dli,dli))
            print "    norm(rzldb) = %e" %blas.nrm2(rzldb)

            rzsdb = [ +hs_k for hs_k in hs ]
            Gs(dx1, rzsdb, alpha=-1.0, beta=dgi)
            misc.cngrnc(r, duzs1, duzs1, trans='T')
            misc.cngrnc(r, duzs1, rzsdb, alpha=1.0, beta=1.0)
            print "    norm(rzsdb) = %e\n" %misc.snrm2(rzsdb)


        def solve_newton1(dx, dy, dzl, dzs, dtau, dsl, dss, dkappa):

            # Solve without refinement
            #
            #     [0     ]   [ 0    A'  Gl'  Gs' 0 ] [dx  ]    [rhsx  ]
            #     [0     ]   [-A    0   0    0   b ] [dy  ]    [rhsy  ]
            #     [dsl   ] - [-Gl   0   0    0   hl] [dzl ] = -[rhszl ]
            #     [dsk   ]   [-Gs   0   0    0   hs] [dzs ]    [rhszs ]
            #     [dkappa]   [-c'  -b' -hl' -hs' 0 ] [dtau]    [rhstau]
            #    
            #     sl.*dzl + zl.*dsl = -rhssl
            #     Hc(dzs*ss + zs*dss) = -rhsss
            #     kappa*dtau + tau*dkappa = -rhskappa
            #
            # Last three equations in scaled variables: 
            #
            #     lmbdl .* (dzl + dsl) = -rhssl
            #     gamma .* (dzs + dss) = -rhsss
            #     lmbdg * (dtau + dkappa) = -rhskappa.
            #
            # On entry, the righthand sides are stored in dx, dy, dzl, 
            # dzs, dtau, dssl, dsss, dkappa.  
            # On exit, scaled quantities are returned for dsl, dzl,
            # dss, dzs.

            if DEBUG:  
                rxdb = xnewcopy(dx)    
                rydb = ynewcopy(dy)
                rzldb = +dzl
                rzsdb = [ +dzs_k for dzs_k in dzs ]
                rtdb = dtau[0]
                rsldb = +dsl
                rssdb = [ +dss_k for dss_k in dss ]
                rkdb = dkappa[0]

            # dy := -dy = -rhsy
            yscal(-1.0, dy) 

            # dsl := -dsl ./ lmbdl   (replace with btmv)
            misc.ihad2(dsl, lmbda, n=ml);  blas.scal(-1,dsl)
            
            # dzl := -(dzl + dsl ./ dli) = -(rhszl - rhssl./zl)  
            blas.axpy(misc.ihad(dsl,dli), dzl);  blas.scal(-1, dzl)  

            # dss := -dss ./ gamma
            for num, den in zip(dss, gamma): misc.ihad2(num, den)
            misc.sscal(-1.0, dss) 

            # dzs := -(dzs + r*dss*r') = -(rhszs - r*(rhss./gamma)*r')
            misc.cngrnc(r, dss, dzs, alpha=-1.0, beta=-1.0)
  
            # dkappa[0] := -dkappa[0]/lmbd[-1] 
            #            = -rhskappa / sqrt(kappa*tau)
            dkappa[0] = -dkappa[0] / lmbda[-1]

            # dtau[0] = dtau[0] + dkappa[0] / dgi
            #         = rhstau[0] - rhskappa / tau
            dtau[0] += dkappa[0] / dgi
 

            # Solve 
            #
            #  [ 0   A'  Gl'  Gs'  0  ][dx  ]   [rhsx                  ]
            #  [-A   0   0    0    b  ][dy  ]   [rhsy                  ]
            #  [-Gl  0   Dl   0    hl ][dzl ] = [rhszl-rhssl./zl       ]
            #  [-Gs  0   0    Ds   hs ][dzs ]   [rhszs-r*(rhszs./ga)*r']
            #  [-c'  -b' -hl' -hs' k/t][dtau]   [rhst-rhsk/tau         ]
            #
            # Dl = diag(dl)^2,  Ds = r*r'*()*r*r'.

            if DEBUG:  
                rxdb2 = xnewcopy(dx)    
                rydb2 = ynewcopy(dy);  xscal(-1, rydb2)
                rzldb2 = -dzl
                rzsdb2 = [-dzs_k for dzs_k in dzs]

                rxdb3 = xnewcopy(dx)    
                rydb3 = ynewcopy(dy);  xscal(-1, rydb3)
                rzldb3 = -dzl
                rzsdb3 = [-dzs_k for dzs_k in dzs]

            f(dx, dy, dzl, dzs)

            if DEBUG: 
                print "second 4x4 system"

                # unscaled variables
                duzl = misc.had(dzl, dli)
                duzs = [ +z_k for z_k in dzs ]
                misc.cngrnc(rti, duzs, duzs)

                A(dy, rxdb2, alpha=-1.0, beta=1.0, trans='T')
                Gl(duzl, rxdb2, alpha=-1.0, beta=1.0, trans='T')
                Gs(duzs, rxdb2, alpha=-1.0, beta=1.0, trans='T')
                print "    norm(rxdb2) = %e" \
                    %math.sqrt(xdot(rxdb2,rxdb2))

                A(dx, rydb2, alpha=1.0, beta=1.0)
                print "    norm(rydb2) = %e" \
                    %math.sqrt(ydot(rydb2,rydb2))

                Gl(dx, rzldb2, alpha=1.0, beta=1.0)
                rzldb2 -= misc.ihad(duzl, misc.had(dli,dli))
                print "    norm(rzldb2) = %e" %blas.nrm2(rzldb2)

                Gs(dx, rzsdb2, alpha=1.0, beta=1.0)
                misc.cngrnc(r, duzs, duzs, trans='T')
                misc.cngrnc(r, duzs, rzsdb2, alpha=-1.0, beta=1.0)
                print "    norm(rzsdb2) = %e\n" %misc.snrm2(rzsdb2)

            dtau[0] = dgi * ( dtau[0] + xdot(c,dx) + ydot(b,dy) + 
                blas.dot(thl, dzl) + misc.sdot(ths, dzs) ) / \
                ( 1.0 + blas.dot(dzl1, dzl1) + misc.sdot(dzs1, dzs1) )
            xaxpy(dx1, dx, alpha=dtau[0])
            yaxpy(dy1, dy, alpha=dtau[0])
            blas.axpy(dzl1, dzl, alpha=dtau[0])
            misc.saxpy(dzs1, dzs, alpha=dtau[0])
 
            # dsl := dsl - dzl = -rhsl ./ lmbdl - dtzl
            blas.axpy(dzl, dsl, alpha=-1)

            # dss := dss - dzs = -rhss ./ Gamma - dzs
            misc.saxpy(dzs, dss, alpha=-1)

            dkappa[0] -= dtau[0]

            if DEBUG: 
                print "solvenewton errors"

                # unscaled variables
                duzl = misc.had(dzl, dli)
                dusl = misc.ihad(dsl, dli)
                duzs = [ +z_k for z_k in dzs ]
                misc.cngrnc(rti, duzs, duzs)
                duss = [ +s_k for s_k in dss ]
                misc.cngrnc(r, duss, duss)
                dutau = dtau[0] * dgi
                dukappa = dkappa[0] / dgi

                print "    norm(rsldb) = %e" %blas.nrm2(rsldb + 
                    misc.had(lmbda[:ml], dzl+dsl))

                for k in xrange(len(ms)):
                    for j in xrange(ms[k]):
                        rssdb[k][j,j+1:] = rssdb[k][j+1:,j].trans() 
                    rssdb[k] += misc.had(dzs[k] + dss[k], gamma[k])
                print "    norm(rssdb) = %e" %misc.snrm2(rssdb)

                print "    abs(rtdb) = %e" %abs(rtdb + dukappa + 
                    xdot(c,dx) + ydot(b,dy) + blas.dot(hl,duzl) + 
                    misc.sdot(hs,duzs))

                A(dy, rxdb, alpha=-1.0, beta=1.0, trans='T')
                Gl(duzl, rxdb, alpha=-1.0, beta=1.0, trans='T')
                Gs(duzs, rxdb, alpha=-1.0, beta=1.0, trans='T')
                xaxpy(c, rxdb, alpha=-dutau)
                print "    norm(rxdb) = %e" %math.sqrt(xdot(rxdb,rxdb))

                A(dx, rydb, alpha=1.0, beta=1.0)
                yaxpy(b, rydb, alpha=-dutau) 
                print "    norm(rydb) = %e" %math.sqrt(ydot(rydb,rydb))

                rzldb += dusl - hl*dutau
                Gl(dx, rzldb, alpha=1.0, beta=1.0)
                print "    norm(rzldb) = %e" %blas.nrm2(rzldb)

                misc.saxpy(duss, rzsdb)
                misc.saxpy(hs, rzsdb, alpha=-dutau)
                Gs(dx, rzsdb, alpha=1.0, beta=1.0)
                print "    norm(rzsdb) = %e\n" %misc.snrm2(rzsdb)

        if refinement:
            def solve_newton(dx, dy, dzl, dzs, dtau, dsl, dss, dkappa):
            
                if DEBUG:  
                    rxdb2 = xnewcopy(dx)    
                    rydb2 = ynewcopy(dy)
                    rzldb2 = +dzl
                    rzsdb2 = [+dzs_k for dzs_k in dzs]
                    rtdb2 = dtau[0]
                    rsldb2 = +dsl
                    rssdb2 = [+dss_k for dss_k in dss]
                    rkdb2 = dkappa[0]

                # copy righthand sides to dx2 etc
                xcopy(dx, dx2)
                ycopy(dy, dy2)
                blas.copy(dzl, dzl2)
                misc.scopy(dzs, dzs2)
                dtau2[0] = dtau[0]
                blas.copy(dsl, dsl2)
                misc.scopy(dss, dss2)
                dkappa2[0] = dkappa[0]
 
                solve_newton1(dx, dy, dzl, dzs, dtau, dsl, dss, dkappa)

                # store residuals in dx2, dy2, etc
                #
                #    [0     ]   [ 0  A'  Gl'  Gs'  c ] [dx  ]   [rhsx  ]
                #    [0     ]   [-A  0   0    0    b ] [dy  ]   [rhsy  ]
                #    [dsl   ] - [-Gl 0   0    0    hl] [dzl ] + [rhszl ]
                #    [dss   ]   [-Gs 0   0    0    hs] [dzs ]   [rhszs ]
                #    [dkappa]   [-c' -b' -hl' -hs' 0 ] [dtau]   [rhstau]
                # 
                #    sl.*dzl + zl.*dsl + rhssl
                #    Hc(dzs*ss + zs*dss) + rhsss
                #    kappa*dtau + tau*dkappa + rhskappa
                #
                # Last 3 equations with scaled variables:
                # 
                #    lmbdl .* (dzl + dsl) + rhssl
                #    0.5 * (diag(lmbds)*dzs + dzs*diag(lmbds) + 
                #        diag(lmbds)*dss + dss*diag(lmbds)) + rhsss
                #    lmbdg * (dtau + dkappa) + rhskappa.
 
                # Store unscaled steps in sl, zl, ss, zs.
                blas.copy(dzl, zl)
                misc.had2(zl, dli)
                blas.copy(dsl, sl)
                misc.ihad2(sl, dli)
                misc.scopy(dzs, zs)
                misc.cngrnc(rti, zs, zs)
                misc.scopy(dss, ss)
                misc.cngrnc(r, ss, ss)
                dutau = dtau[0] * dgi
                dukappa = dkappa[0] / dgi

                A(dy, dx2, alpha=-1.0, beta=1.0, trans='T')
                Gl(zl, dx2, alpha=-1.0, beta=1.0, trans='T')
                Gs(zs, dx2, alpha=-1.0, beta=1.0, trans='T')
                xaxpy(c, dx2, alpha=-dutau)

                A(dx, dy2, alpha=1.0, beta=1.0)
                yaxpy(b, dy2, alpha=-dutau)

                Gl(dx, dzl2, alpha=1.0, beta=1.0)
                blas.axpy(hl, dzl2, alpha=-dutau)
                blas.axpy(sl, dzl2)

                Gs(dx, dzs2, alpha=1.0, beta=1.0) 
                misc.saxpy(hs, dzs2, alpha=-dutau)
                misc.saxpy(ss, dzs2)

                dtau2[0] += dukappa + xdot(c,dx) + ydot(b,dy) + \
                    blas.dot(hl, zl) + misc.sdot(hs, zs)

                dsl2[:] += misc.had(dzl+dsl, lmbda[:ml]) 

                for k in xrange(len(ms)):
                    dss2[k] += misc.had(dzs[k]+dss[k], gamma[k])

                dkappa2[0] += lmbda[-1] * (dtau[0] + dkappa[0])

                solve_newton1(dx2, dy2, dzl2, dzs2, dtau2, dsl2, dss2, 
                    dkappa2)

                xaxpy(dx2, dx)
                yaxpy(dy2, dy)
                blas.axpy(dzl2, dzl)
                misc.saxpy(dzs2, dzs)
                dtau[0] += dtau2[0]
                blas.axpy(dsl2, dsl)
                misc.saxpy(dss2, dss)
                dkappa[0] += dkappa2[0]

                if DEBUG: 
                    print "solvenewton2 errors"

                    # unscaled variables
                    duzl = misc.had(dzl, dli)
                    dusl = misc.ihad(dsl, dli)
                    duzs = [ +z_k for z_k in dzs ]
                    misc.cngrnc(rti, duzs, duzs)
                    duss = [ +s_k for s_k in dss ]
                    misc.cngrnc(r, duss, duss)
                    dutau = dtau[0]*dgi
                    dukappa = dkappa[0]/dgi

                    print "    norm(rsldb2) = %e" %blas.nrm2(rsldb2 + 
                        misc.had(lmbda[:ml],dzl+dsl))

                    for k in xrange(len(ms)):
                        for j in xrange(ms[k]): rssdb2[k][j,j+1:] = \
                            rssdb2[k][j+1:,j].trans()
                        rssdb2[k] += misc.had(dzs[k] + dss[k], gamma[k])
                    print "    norm(rssdb2) = %e" %misc.snrm2(rssdb2)

                    print "    abs(rtdb2) = %e" %abs(rtdb2 + dukappa 
                        + xdot(c,dx) + ydot(b,dy) + blas.dot(hl,duzl) 
                        + misc.sdot(hs,duzs))

                    A(dy, rxdb2, alpha=-1.0, beta=1.0, trans='T')
                    Gl(duzl, rxdb2, alpha=-1.0, beta=1.0, trans='T')
                    Gs(duzs, rxdb2, alpha=-1.0, beta=1.0, trans='T')
                    xaxpy(c, rxdb2, alpha=-dutau)
                    print "    norm(rxdb2) = %e" \
                        %math.sqrt(xdot(rxdb2,rxdb2))

                    A(dx, rydb2, alpha=1.0, beta=1.0)
                    yaxpy(b, rydb2, alpha=-dutau) 
                    print "    norm(rydb2) = %e" \
                        %math.sqrt(ydot(rydb2,rydb2))
                    rzldb2 += dusl - hl*dutau
                    Gl(dx, rzldb2, alpha=1.0, beta=1.0)
                    print "    norm(rzldb2) = %e" %blas.nrm2(rzldb2)

                    misc.saxpy(duss, rzsdb2)
                    misc.saxpy(hs, rzsdb2, alpha=-dutau)
                    Gs(dx, rzsdb2, alpha=1.0, beta=1.0)
                    print "    norm(rzsdb2) = %e\n" %misc.snrm2(rzsdb2)

        else:
            solve_newton = solve_newton1

        mu = blas.dot(lmbda,lmbda) / (m+1)

        # Affine scaling step 
        blas.copy(lmbda, dsl, n=ml)        
        misc.had2(dsl,lmbda, n=ml)        
        misc.sscal(0.0, dss) 
        ind = ml
        for e, mk in zip(dss, ms):
             blas.copy(lmbda, e, n=mk, offsetx=ind, incy=mk+1)
             blas.tbmv(lmbda, e, n=mk, k=0, ldA=1, offsetA=ind, 
                 incx=mk+1)
             ind += mk
        dkappa[0] = kappa*tau
        xcopy(resx, dx)
        ycopy(resy, dy)
        blas.copy(reszl, dzl)
        misc.scopy(reszs, dzs)
        dtau[0] = rest
        solve_newton(dx, dy, dzl, dzs, dtau, dsl, dss, dkappa)

        # Maximum step to boundary
        blas.copy(dzl, sigz)
        misc.ihad2(sigz, lmbda, n=ml) 
        blas.copy(dsl, sigs)
        misc.ihad2(sigs, lmbda, n=ml) 
        misc.scopy(dss, ss)
        misc.scopy(dzs, zs)
        ind = ml
        for k in xrange(len(ms)):
            for i in xrange(ms[k]):
                a = 1.0 / math.sqrt(lmbda[ind+i])
                blas.scal(a, ss[k], offset=i, n=i+1, inc=ms[k])
                blas.scal(a, ss[k], offset=i*ms[k]+i, n=ms[k]-i)
                blas.scal(a, zs[k], offset=i, n=i+1, inc=ms[k])
                blas.scal(a, zs[k], offset=i*ms[k]+i, n=ms[k]-i)
            lapack.syevd(ss[k], sigs, offsetW=ind, jobz='V')  
            lapack.syevd(zs[k], sigz, offsetW=ind, jobz='V')  
            ind += ms[k]
        sigz[-1], sigs[-1] = dtau[0]/lmbda[-1], dkappa[0]/lmbda[-1]
        step = min(1.0, STEP / max(-min(sigz), -min(sigs))) 

        newmu = ( (1-step) * blas.dot(lmbda,lmbda) + step**2 * 
            (blas.dot(dsl, dzl) + misc.sdot(dss, dzs) + 
            dkappa[0]*dtau[0]) ) / ( m+1)
        sigma = min(1.0, (newmu/mu)**EXPON)

        # Centering-corrector step
        misc.had2(dsl, dzl)
        blas.axpy(misc.had(lmbda[:ml], lmbda[:ml]), dsl)
        blas.axpy(ones, dsl, alpha=-sigma*mu, n=ml)
        ind = ml
	for k in xrange(len(ms)):
            # work := dss[k] with upper triangular part
            for i in xrange(ms[k]-1):
                dss[k][i,i+1:] = dss[k][i+1:,i].trans()
                dzs[k][i,i+1:] = dzs[k][i+1:,i].trans()
            blas.copy(dss[k], work)

            # dss[k] := 0.5 * (work*dzs[k] + dzs[k]*work)
            #     = 0.5 * (dss[k]*dzs[k] + dzs[k]*dss[k])
            blas.syr2k(work, dzs[k], dss[k], n=ms[k], k=ms[k], 
                ldA=ms[k], alpha=0.5)

            # dss[k] := dss[k] + diag(lambda)^2 - sigma*mu*I
            blas.copy(lmbda, work, n=ms[k], offsetx=ind)
            blas.tbmv(lmbda, work, n=ms[k], k=0, ldA=1, offsetA=ind)
            blas.axpy(work, dss[k], incy=ms[k]+1, n=ms[k])
            blas.axpy(ones, dss[k], alpha=-sigma*mu, incy=ms[k]+1, 
                n=ms[k])

            ind += ms[k]

        dkappa[0] = lmbda[-1]**2 + dkappa[0]*dtau[0] - sigma*mu
        xcopy(resx, dx);  xscal(1.0-sigma, dx)
        ycopy(resy,dy);  yscal(1.0-sigma, dy)
        blas.copy(reszl, dzl);  blas.scal(1.0-sigma, dzl)
        misc.scopy(reszs, dzs); misc.sscal(1.0-sigma, dzs)
        dtau[0] = (1.0-sigma)*rest 
        solve_newton(dx, dy, dzl, dzs, dtau, dsl, dss, dkappa)

        # Maximum step to boundary
        blas.copy(dzl, sigz)
        misc.ihad2(sigz, lmbda, n=ml) 
        blas.copy(dsl, sigs)
        misc.ihad2(sigs, lmbda, n=ml) 
        ind = ml
        for k in xrange(len(ms)):
            for i in xrange(ms[k]):
                a = 1.0 / math.sqrt(lmbda[ind+i])
                blas.scal(a, dss[k], offset=i, n=i+1, inc=ms[k])
                blas.scal(a, dss[k], offset=i*(ms[k]+1), n=ms[k]-i)
                blas.scal(a, dzs[k], offset=i, n=i+1, inc=ms[k])
                blas.scal(a, dzs[k], offset=i*(ms[k]+1), n=ms[k]-i)
            lapack.syevd(dss[k], sigs, offsetW=ind, jobz='V')  
            lapack.syevd(dzs[k], sigz, offsetW=ind, jobz='V')  
            ind += ms[k]
        sigz[-1], sigs[-1] = dtau[0]/lmbda[-1], dkappa[0]/lmbda[-1]
        step = min(1.0, STEP / max(-min(sigz), -min(sigs))) 

        # Update lmbdl, dli
        dli = misc.ihad(dli, base.sqrt(misc.ihad(lmbda[:ml] + step*dsl, 
            lmbda[:ml] + step*dzl)))
        lmbda[:ml] = base.sqrt(misc.had(lmbda[:ml]+step*dsl, 
            lmbda[:ml]+step*dzl))

        dgi = dgi * math.sqrt( (lmbda[-1] + step*dtau[0]) / 
            (lmbda[-1] + step*dkappa[0]) )
        lmbda[-1] =  math.sqrt( (lmbda[-1] + step*dtau[0]) *  
            (lmbda[-1] + step*dkappa[0]) )  

        # Update scaling matrices, lmbdas, gamma.
        ind = ml
        for k in xrange(len(ms)):
            for i in xrange(ms[k]):
                # dss[k] := L1 
                #     = lmbdas[k]^{1/2} * dss[k] * (1+step*sigs)^{1/2}
                # dzs[k] := L2 
                #     = lmbdas[k]^{1/2} * dzs[k] * (1+step*sigz)^{1/2}
                a = math.sqrt(lmbda[ind+i]) 
                blas.scal(a, dss[k], offset=i, inc=ms[k], n=ms[k])
                blas.scal(a, dzs[k], offset=i, inc=ms[k], n=ms[k])
                blas.scal(math.sqrt(1+step*sigs[ind+i]), dss[k], 
                    offset=ms[k]*i, n=ms[k])
                blas.scal(math.sqrt(1+step*sigz[ind+i]), dzs[k], 
                    offset=ms[k]*i, n=ms[k])

            # r[k] := r[k]*dss[k] = r[k]*L1[k]
            # rti[k] := rti[k]*dzs[k] = rti[k]*L2[k]
            blas.gemm(r[k], dss[k], work, ldC=ms[k])
            blas.copy(work, r[k], n=ms[k]**2)
            blas.gemm(rti[k], dzs[k], work, ldC=ms[k])
            blas.copy(work, rti[k], n=ms[k]**2)

            # SVD of L2'*L1 = U*lmbds^+*V', store U in dss[k] and V' in
            # dzs[k]
            blas.gemm(dzs[k], dss[k], work, transA='T', ldC=ms[k])
            lapack.gesvd(work, lmbda, jobu='A', jobvt='A', m=ms[k],
                n=ms[k], ldA=ms[k], U=dss[k], Vt=dzs[k], offsetS=ind)

            # r[k] := r[k]*V,  rti[k] := rti[k]*U
            blas.gemm(r[k], dzs[k], work, transB='T', ldC=ms[k])
            blas.copy(work, r[k], n=ms[k]**2)
            blas.gemm(rti[k], dss[k], work, ldC=ms[k])
            blas.copy(work, rti[k], n=ms[k]**2)

            # r[k] := r[k]*lambda^{-1/2}; rti[k] := rti[k]*lambda^{-1/2}
            for i in xrange(ms[k]):    
                a = 1.0 / math.sqrt(lmbda[ind+i])
                blas.scal(a, r[k], offset=ms[k]*i, n=ms[k])
                blas.scal(a, rti[k], offset=ms[k]*i, n=ms[k])

            # gamma[k][i,j] = .5 * (lambda_i + lambda_j)
            # Use blas.ger to make sure there are no zeros or nan's in 
            # upper triangular part.

            blas.scal(0.0, gamma[k])
            blas.ger(lmbda, ones, gamma[k], alpha=.5, offsetx=ind)
            blas.ger(ones, lmbda, gamma[k], alpha=.5, offsety=ind)

            ind += ms[k]


        # Compute new sl, zl, ss, zs, tau, kappa (used to evaluate the
        # residuals).

        # sl := lmbdl ./ dli  
        blas.copy(lmbda, sl, n=ml)
        misc.ihad2(sl, dli) 

        # zl = lmbdl .* dli
        blas.copy(lmbda, zl, n=ml)
        misc.had2(zl, dli)

        kappa, tau = lmbda[-1]/dgi, lmbda[-1]*dgi

        # ss = r*lambda*r',  zs = rti*lambda*rti'
        ind = ml
        for k in xrange(len(ms)):
            blas.copy(r[k], work)
            blas.copy(rti[k], work2)
            for i in xrange(ms[k]):
                a = math.sqrt(lmbda[ind+i])
                blas.scal(a, work, offset=ms[k]*i, n=ms[k])
                blas.scal(a, work2, offset=ms[k]*i, n=ms[k])
            blas.syrk(work, ss[k], ldA=ms[k], n=ms[k], k=ms[k])
            blas.syrk(work2, zs[k], ldA=ms[k], n=ms[k], k=ms[k])
            ind += ms[k]

        xaxpy(dx, x, alpha=step)
        yaxpy(dy, y, alpha=step)

        gap = blas.dot(lmbda, lmbda, n=m-1) / tau**2


    return {'status': 'unknown', 'x': None, 'y': None, 'sl': None, 
        'ss': None, 'zl': None, 'zs': None}



def lp(c, G, h, A=None, b=None, solver=None, primalstart=None,
    dualstart=None):
    """

    Solves a pair of primal and dual LPs

        minimize    c'*x             maximize    -h'*z - b'*y 
        subject to  G*x + s = h      subject to  G'*z + A'*y + c = 0
                    A*x = b                      z >= 0
                    s >= 0

    Input arguments:

        G is mxn, h is mx1, A is pxn, b is px1.  G and A must be dense
        or sparse 'd' matrices.   h and b are dense 'd' matrices with 
        one column.  The default values for A and b are empty matrices 
        with zero rows.

        solver is None, 'glpk' or 'mosek'.  The default solver (None)
        uses an LP solver implemented in Python.  The 'glpk' solver is 
        the simplex LP solver from GLPK.  The 'mosek' solver is the LP 
        solver from MOSEK.

        The arguments primalstart and dualstart are ignored when solver
        is 'glpk' or 'mosek' and are optional when solver is None.  
        The argument primalstart is a dictionary with keys 'x' and 's'
        and specifies a primal starting point.  primalstart['x'] must 
        be a dense 'd' matrix of length n;  primalstart['s'] must be a 
        positive dense 'd' matrix of length m.
        The argument dualstart is a dictionary with keys 'z' and 'y' 
        and specifies a dual starting point.   dualstart['y'] must 
        be a dense 'd' matrix of length p;  dualstart['z'] must be a 
        positive dense 'd' matrix of length m.

        When solver is None, we require n >= 1, Rank(A) = p and 
        Rank([G; A]) = n


    Returns a dictionary with keys 'status', 'x', 's', 'y', 'z'.

        If solver is None or 'mosek':
        
            If status is 'optimal', x, s, y, z are the primal and dual 
            optimal solutions.

            If status is 'primal infeasible', x = s = None, and z, y 
            are a proof of infeasibility: 

                h'*z + b'*y = -1,  G'*z + A'*y = 0,  z >= 0.

            If status is 'dual infeasible', z = y = None, and x, s are 
            a proof of dual infeasibility: 

                c'*x = -1,  G*x + s = 0,  A*x = 0,  s >= 0.

            If status is 'unknown', x, y, s, z are None.


        If solver is 'glpk':
        
            If status is 'optimal', x, s, y, z are the primal and dual 
            optimal solutions.

            If status is 'primal infeasible', 'dual infeasible',
            or 'unknown', then x = s = z = y = None.


    The control parameters for the different solvers can be modified by 
    adding an entry to the dictionary cvxopt.solvers.options.  The 
    following parameters control the execution of the default solver.

        options['show_progress'] True/False (default: True)
        options['maxiters'] positive integer (default: 100)
        options['abstol'] scalar (default: 1e-7)
        options['reltol'] scalar (default: 1e-7)
        options['feastol'] scalar (default: 1e-7)
        options['refinement'] True/False (default: True)

    The control parameter names for GLPK and MOSEK can be found in the
    GLPK and MOSEK documentation.  Options that are not recognized are 
    replaced by their default values.
    """

    if type(c) is not matrix or c.typecode != 'd' or c.size[1] != 1: 
        raise TypeError, "'c' must be a dense column matrix"
    n = c.size[0]
    if n < 1: raise ValueError, "number of variables must be at least 1"

    if (type(G) is not matrix and type(G) is not spmatrix) or \
        G.typecode != 'd' or G.size[1] != n:
        raise TypeError, "'G' must be a dense or sparse 'd' matrix "\
            "with %d columns" %n 
    m = G.size[0]
    if type(h) is not matrix or h.typecode != 'd' or h.size != (m,1):
        raise TypeError, "'h' must be a 'd' matrix of size (%d,1)" %m

    if A is None:  A = spmatrix([], [], [], (0,n), 'd')
    if (type(A) is not matrix and type(A) is not spmatrix) or \
        A.typecode != 'd' or A.size[1] != n:
        raise TypeError, "'A' must be a dense or sparse 'd' matrix "\
            "with %d columns" %n
    p = A.size[0]
    if b is None: b = matrix(0.0, (0,1))
    if type(b) is not matrix or b.typecode != 'd' or b.size != (p,1): 
        raise TypeError, "'b' must be a dense matrix of size (%d,1)" %p

    if solver == 'glpk':
        try: from cvxopt import glpk
        except ImportError: raise ValueError, "invalid option "\
            "(solver='glpk'): cvxopt.glpk is not installed" 
        glpk.options = options
        status, x, z, y = glpk.solvelp(c,G,h,A,b)
        if status == 'optimal':
            s = matrix(h)
            base.gemv(G, x, s, alpha=-1.0, beta=1.0)
        else:
            s = None
        return {'status': status, 'x': x, 's': s, 'y': y, 'z': z}

    if solver == 'mosek':
        try: from cvxopt import mosek
        except ImportError: raise ValueError, "invalid option "\
            "(solver='mosek'): cvxopt.mosek is not installed" 
        mosek.options = options
        prosta, solsta, x, z, y  = mosek.solvelp(c,G,h,A,b)
        if solsta == 'OPTIMAL':
            s = matrix(h)
            base.gemv(G, x, s, alpha=-1.0, beta=1.0)
            status = 'optimal'
        elif solsta == 'PRIMAL_INFEASIBLE_CER':
            status = 'primal infeasible'
            ducost = -blas.dot(h,z) - blas.dot(b,y)
            blas.scal(1.0/ducost, y)
            blas.scal(1.0/ducost, z)
            x, s = None, None
        elif solsta == 'DUAL_INFEASIBLE_CER':
            status = 'dual infeasible'
            pcost = blas.dot(c,x)
            blas.scal(-1.0/pcost, x)
            s = matrix(0.0, (m,1))
            base.gemv(G, x, s, alpha=-1.0)
            z, y = None, None
        else: 
            status = 'unknown'
            x, s, y, z = None, None, None, None
        return {'status': status, 'x': x, 's': s, 'y': y, 'z': z}

    if p > n or n-p > m:
        raise ValueError, "Rank(A) < p or Rank([G; A]) < n"

    def Fi(x, y, alpha=1.0, beta=0.0, trans='N'):
        base.gemv(G, x, y, alpha=alpha, beta=beta, trans=trans)

    def Fe(x, y, alpha=1.0, beta=0.0, trans='N'):
        base.gemv(A, x, y, alpha=alpha, beta=beta, trans=trans)

    def Fs(x, y, alpha=1.0, beta=0.0, trans='N'): 
        pass
    

    # kktsolver(di, rti) returns a function for solving 
    #
    #    [ 0  A'  G'             ] [x]   [bx]
    #    [ A  0   0              ] [y] = [by].
    #    [ G  0   -diag(di)^{-2} ] [z]   [bz]
    #
    # di is a positive mx1 'd' matrix;  the argument rti is ignored. 
    #
    # We return the scaled z: z./di instead of z.

    global kktmethod
    try: kktmethod = options['kktmethod']
    except KeyError: kktmethod = 3
    else: 
        if type(kktmethod) is not int or kktmethod not in [1,2,3]:
            raise ValueError, "options['kktmethod'] must be 1, 2, or 3"

    if kktmethod == 1:

        # The entire matrix is factored using a dense LDL factorization.

        K = matrix(0.0, (n+p+m,n+p+m))
        ipiv = matrix(0, (n+p+m,1))
        u = matrix(0.0, (n+p+m,1))
        def kktsolver(di, rti):
            blas.scal(0, K)
            K[n:n+p,:n],  K[n+p:,:n] = A, G  
            K[(m+p+n+1)*(p+n)::n+m+p+1] = -misc.ihad(1.0, 
                misc.had(di, di))
            lapack.sytrf(K, ipiv)
            def solve_kkt(x, y, z, zs):
		blas.copy(x, u)
		blas.copy(y, u, offsety=n)
		blas.copy(z, u, offsety=n+p)
		lapack.sytrs(K, ipiv, u)
		blas.copy(u, x, n=n)
		blas.copy(u, y, offsetx=n, n=p)
		blas.copy(u, z, offsetx=n+p, n=m)
                misc.ihad2(z, di)
            return solve_kkt

    else:
        if type(G) is matrix: 
            Gsc = matrix(0.0, (m,n)) 
        else:
            Gsc = spmatrix(0.0, G.I, G.J, (m,n)) 

        def GDG(Gsc, S, di, pattern_S_is_correct=True):  

            # Compute 
            #
            #    Gsc := diag(di)*G and 
            #    S[:n,:n] := G' * diag(di)^2 * G.
            #
            # If G is dense, Gsc and S must be dense.  If G is sparse, 
            # Gsc is sparse and S may be dense or sparse.

            base.gemm(spmatrix(di, range(m), range(m), tc='d'), G, 
                Gsc, partial=True)
            base.syrk(Gsc, S, trans='T', partial=pattern_S_is_correct)

        if kktmethod == 2:

            # We compute x, y, z by solving
            #
            #    [G'*diag(di)^2*G  A'] [x]   [bx + G'*(di.^2).*bz)]
            #    [                   ]*[ ] = [                    ]
            #    [A                0 ] [y]   [by                  ]
            #
            #    z = (di.^2) .* (G*x - bz).
            #    
            # We return the scaled z:  z = di .* (G*x - bz).
            #
            # The coefficient matrix is factored using a dense LDL 
            # (if p>0) or dense Cholesky factorization (if p=0).  

            K = matrix(0.0, (n+p,n+p))
            if p: ipiv = matrix(0, (n+p,1))
            u = matrix(0.0, (n+p,1))
            def kktsolver(di, rti):
                blas.scal(0.0, K)
                K[n:,:n] = A
                GDG(Gsc, K, di)
                if p: lapack.sytrf(K, ipiv)
                else: lapack.potrf(K)
                def solve_kkt(x, y, z, zs):
                    blas.copy(x, u)
                    blas.copy(y, u, offsety=n)
                    misc.had2(z, di)
                    base.gemv(Gsc, z, u, trans='T', beta=1.0)
                    if p: lapack.sytrs(K, ipiv, u)
                    else: lapack.potrs(K, u)
                    blas.copy(u, x, n=n)
                    blas.copy(u, y, n=p, offsetx=n)
                    base.gemv(Gsc, x, z, beta=-1.0)
                return solve_kkt

        else:

            # Solve 
            #
            #    K*y = A * S^{-1} * (bx + G'*((di.^2).*bz)) - by
            #    S*x = bx + G'*((di.^2).*bz) - A'*y
            #    z = (di.^2) .* (G*x - bz).
            #
            # where K = A*S^{-1}*A', S = G'*diag(di)^2*G.
            #
            # If in the first call to kktsolver, S is singular, switch
            # to kktmethod 4, i.e., solve
            #
            #    K*y = A * S^{-1} * (bx + G'*((di.^2).*bz) + A'*by) - by
            #    S*x = bx + G'*((di.^2).*bz) + A'*by - A'*y
            #    z = (di.^2) .* (G*x - bz).
            #
            # where K = A'*S^{-1}*A, S = G'*diag(di)^2*G + A'*A
            # 
            # The matrices K and S are factored using dense or sparse 
            # Cholesky factorizations.
            #
            # We return the scaled z:  z = di .* (G*x - bz).

            global S, Sf, firstcall
            firstcall = True

            if type(G) is matrix:
                S = matrix(0.0, (n,n))
                K = matrix(0.0, (p,p))
            else:
                S = spmatrix([], [], [], (n,n), 'd')
                if type(A) is matrix: K = matrix(0.0, (p,p))
                else: K = spmatrix([], [], [], (p,p), tc='d')

            def factorS(di,S):

                # Factor S = G'*diag(di)^2*G if kktmethod = 3.
                # Factor S = G'*diag(di)^2*G + A'*A if kktmethod = 4.
                # If S is sparse, store factorization in Sf.

                global Sf
                GDG(Gsc, S, di, not firstcall)
                if kktmethod == 4:
                    base.syrk(A, S, trans='T', beta=1.0, partial=not
                        firstcall) 
                if type(S) is matrix:
                    lapack.potrf(S) 
                else:
                    if firstcall: Sf = cholmod.symbolic(S)
                    cholmod.numeric(S, Sf)

            def kktsolver(di, rti):
                global firstcall, kktmethod, S
                if kktmethod == 3:
                    try: factorS(di,S)
                    except ArithmeticError:
                        if firstcall:
                            kktmethod = 4
                            if type(S) is spmatrix and type(A) is \
                                matrix: S = matrix(0.0, (n,n))
                            factorS(di,S)
                        else: raise ArithmeticError
                else: 
                    factorS(di,S)
                firstcall = False

                if type(S) is matrix:
                    # Asct := L^{-1}*A',  factor K = Asct'*Asct
                    if type(A) is matrix:
                        Asct = A.trans()
                    else:
                        Asct = matrix(A.trans())
                    blas.trsm(S, Asct)
                    blas.syrk(Asct, K, trans='T')
                    lapack.potrf(K)

                else:
                    # Asct := L^{-1}*P*A' and factor K = Asct'*Asct.
                    if type(A) is matrix:
                        Asct = A.trans()
                        cholmod.solve(Sf, Asct, sys=7)
                        cholmod.solve(Sf, Asct, sys=4)
                        blas.syrk(Asct, K, trans='T')
                        lapack.potrf(K) 
                    else:
                        Asct = cholmod.spsolve(Sf, A.trans(), sys=7)
                        Asct = cholmod.spsolve(Sf, Asct, sys=4)
                        base.syrk(Asct, K, trans='T')
                        Kf = cholmod.symbolic(K)
                        cholmod.numeric(K, Kf)

                def solve_kkt(x, y, z, zs):

                    # If kktmethod is 3:
                    # x := L^{-1}*P * (x + G'*((di.^2).*z)) 
                    #    = L^{-1}*P * (bx + G'*((di.^2).*bz))
                    #
                    # If kktmethod is 4:
                    # x := L^{-1}*P * (x + G'*((di.^2).*z) + A'*y)
                    #    = L^{-1}*P * (bx + G'*((di.^2).*bz) + A'*by)
                    misc.had2(z, di)
                    base.gemv(Gsc, z, x, trans='T', beta=1.0)
                    if kktmethod == 4:
                        base.gemv(A, y, x, trans='T', beta=1.0)
                    if type(S) is matrix:
                        blas.trsv(S, x)
                    else:
                        cholmod.solve(Sf, x, sys=7)
                        cholmod.solve(Sf, x, sys=4)

		    # y := K^{-1} * (Asc*x - y)
                    #    = K^{-1} * (A*S^{-1}* (bx + G'*((di.^2).*bz)) 
                    #      - by)  (kktmethod = 3)
                    #    = K^{-1} * (A*S^{-1}* (bx + G'*((di.^2).*bz))
                    #      + A'*by) - by) (kktmethod = 4)
		    base.gemv(Asct, x, y, trans='T', beta=-1.0)
		    if type(K) is matrix:
                        lapack.potrs(K, y)
                    else:
                        cholmod.solve(Kf, y)

                    # x := P' * L^{-T} * (x - Asc'*y)
                    #    = S^{-1}* (bx + G'*((di.^2).*bz) - A'*y)
                    #      (kktmethod = 3)  
                    #    = S^{-1}*(bx + G'*((di.^2).*bz) + A'*by - 
                    #      A'*y) (kktmethod = 4)
		    base.gemv(Asct, y, x, alpha=-1.0, beta=1.0)
                    if type(S) is matrix:
                        blas.trsv(S, x, trans='T')
                    else:
                        cholmod.solve(Sf, x, sys=5)
                        cholmod.solve(Sf, x, sys=8)

		    # z := Gsc*x - z = di .* (G*x - bz)
                    base.gemv(Gsc, x, z, beta=-1.0)

                return solve_kkt 

    if primalstart:
        pst = {'x': primalstart['x'], 'sl': primalstart['s'], 'ss': []}
    else: 
        pst = None
    if dualstart:
        dst = {'y': dualstart['y'], 'zl': dualstart['z'], 'zs': []}
    else: 
        dst = None
    sol = conelp(c, kktsolver, Gl=Fi, hl=h, A=Fe, b=b, primalstart=pst,
        dualstart=dst)
    return {'status': sol['status'], 'x': sol['x'], 'y': sol['y'],
         'z': sol['zl'], 's': sol['sl']}



def sdp(c, Gl=None, hl=None, Gs=None, hs=None, A=None, b=None, 
    solver=None, primalstart=None, dualstart=None):
    """

    Solves a pair of primal and dual SDPs

        minimize    c'*x             
        subject to  Gl*x + sl = hl      
                    Gs(x) + ss = hs
                    A*x = b                      
                    sl >= 0,  ss >= 0

        maximize    -hl'*z - tr(hs,zs) - b'*y
        subject to  Gl'*zl + Gs'(zs) + A'*y + c = 0
                    zl >= 0,  zs >= 0

    Input arguments:

        Gl is an mlxn dense or sparse 'd' matrix.  
        hl is an mlx1 dense 'd' matrix.  The default values of Gl and 
        hl are matrices with zero rows.

        The argument Gs is a list of N dense or sparse 'd' matrices of 
        size (ms[k]**2, n), k=1,...,N. 
        The matrix-vector products Gs[k]*x, k=1,...,N, represent a 
        block-diagonal symmetric matrix with N diagonal blocks of size 
        (ms[k],ms[k]), stored columnwise as vectors.   
        hs is a list of dense 'd' matrices of size (ms[k],ms[k]).  
        Only the lower-triangular part of hs[k] is accessed.

        A is a pxn dense or sparse 'd' matrix.
        b is a px1 dense 'd' matrix.  The default values of A and b are 
        matrices with zero rows.

        solver is None or 'dsdp'.  The default solver (None) uses an 
        SDP solver implemented in Python.  The 'dsdp' solver uses an
        interface to DSDP5.  It does not accept problems with equality
        constraints (A and b must have zero rows, or be absent).

        The argument primalstart is a dictionary with keys 'x', 'sl',
        'ss', and specifies a primal starting point.  primalstart['x'] 
        must be a dense 'd' matrix of length n;  primalstart['sl'] must
        be a positive dense 'd' matrix of length ml;  primalstart['ss']
        must be a list of positive definite matrices of size 
        ms[k]xms[k].
        The argument dualstart is a dictionary with keys 'zl', 'zs', 
        'y' and specifies a dual starting point.   primalstart['y'] 
        must be a dense 'd' matrix of length p;  primalstart['zl'] 
        must be a positive dense 'd' matrix of length ml;  
        primalstart['zs'] must be a list of positive definite matrices 
        of size (ms[k],ms[k]).
        The arguments primalstart and dualstart are ignored when solver
        is 'dsdp'.

    Returns a dictionary with keys 'status', 'x', 'sl', 'ss', 'y', 'zl',
        'zs'.

        If status is 'optimal', x, sl, ss, y, zl, zs are the primal and
        dual optimal solutions.

        If status is 'primal infeasible', x = sl = ss = None, and zl, 
        zs, y are a proof of infeasibility: 

            hl'*zl + tr(hs,zs) + b'*y = -1,  
            Gl'*zl + Gs'(zs) + A'*y = 0,  
            zl >= 0, zs >= 0.

        If status is 'dual infeasible', zl = zs = y = None, and x, sl, 
        ss are a proof of dual infeasibility: 

            c'*x = -1,  Gl*x + sl = 0,  Gs(x) + ss = 0,  A*x = 0,  
            sl >= 0, ss >= 0.

        If status is 'unknown', x, y, sl, ss, zl, zs are None.

    The following parameters control the execution of the default 
    solver.

        options['show_progress'] True/False (default: True)
        options['maxiters'] positive integer (default: 100)
        options['abstol'] scalar (default: 1e-7)
        options['reltol'] scalar (default: 1e-7)
        options['feastol'] scalar (default: 1e-7)
        options['refinement'] True/False (default: True).

    The execution of the 'dsdp' solver is controlled by: 

        options['show_progress'] integer (default: 0)
        options['maxiters'] positive integer 
        options['rgaptol'] scalar (default: 1e-5).
    """

    if type(c) is not matrix or c.typecode != 'd' or c.size[1] != 1: 
        raise TypeError, "'c' must be a dense column matrix"
    n = c.size[0]
    if n < 1: raise ValueError, "number of variables must be at least 1"

    if Gl is None: Gl = spmatrix([], [], [], (0,n), tc='d')
    if (type(Gl) is not matrix and type(Gl) is not spmatrix) or \
        Gl.typecode != 'd' or Gl.size[1] != n:
        raise TypeError, "'Gl' must be a dense or sparse 'd' matrix "\
            "with %d columns" %n 
    ml = Gl.size[0]
    if hl is None: hl = matrix(0.0, (0,1))
    if type(hl) is not matrix or hl.typecode != 'd' or \
        hl.size != (ml,1):
        raise TypeError, "'hl' must be a 'd' matrix of size (%d,1)" %ml

    if Gs is None: Gs = []
    if type(Gs) is not list or [ Gs_k for Gs_k in Gs if (type(Gs_k) is 
        not matrix and type(Gs_k) is not spmatrix) or 
        Gs_k.typecode != 'd' or Gs_k.size[1] != n ]:
        raise TypeError, "'Gs' must be a list of sparse or dense 'd' "\
            "matrices with %d columns" %n 
    ms = [ int(math.sqrt(Gs_k.size[0])) for Gs_k in Gs ]
    maxm, N = max([0] + ms), len(ms)
    a = [ k for k in xrange(len(Gs)) if ms[k]**2 != Gs[k].size[0] ]
    if a: raise TypeError, "the squareroot of the number of rows in "\
        "'Gs[%d]' is not an integer" %k
    if hs is None: hs = []
    if type(hs) is not list or len(hs) != N or [ k for k in xrange(N) 
        if type(hs[k]) not in (matrix, spmatrix) or 
        hs[k].typecode != 'd' or hs[k].size != (ms[k],ms[k]) ]:
        raise TypeError, "'hs' must be a list of %d dense or sparse "\
            "'d' matrices of size (%d,%d)" %(N,ms[k],ms[k])

    if A is None: A = spmatrix([], [], [], (0,n), 'd')
    if (type(A) is not matrix and type(A) is not spmatrix) or \
        A.typecode != 'd' or A.size[1] != n:
        raise TypeError, "'A' must be a dense or sparse 'd' matrix "\
            "with %d columns" %n
    p = A.size[0]
    if b is None: b = matrix(0.0, (0,1))
    if type(b) is not matrix or b.typecode != 'd' or b.size != (p,1): 
        raise TypeError, "'b' must be a dense matrix of size (%d,1)" %p

    if p > n or n-p > ml+ sum([mk*(mk+1)/2 for mk in ms]):
        raise ValueError, "Rank(A) < p or Rank([Gl; Gs; A]) < n"

    if not sum(ms):   # call lp() 
        if primalstart is not None:
            ps = primalstart.copy()
            if ps.has_key('sl'): ps['s'] = ps['sl']
        else: 
            ps = None
        if dualstart is not None:
            ds = dualstart.copy()
            if ds.has_key('zl'): ds['z'] = ds['zl']
        else:
            ds = None
        sol = lp(c, Gl, hl, A, b, solver=solver, primalstart=ps, 
            dualstart=ds)
        sol['sl'] = sol['s'];  del sol['s']  
        sol['ss'] = [ matrix(0.0, (0,0)) for k in xrange(N)] 
        sol['zl'] = sol['z'];  del sol['z']  
        sol['zs'] = [ matrix(0.0, (0,0)) for k in xrange(N)];  
        return sol

    if solver == 'dsdp':
        try: from cvxopt import dsdp
        except ImportError: raise ValueError, "invalid option "\
            "(solver = 'dsdp'): cvxopt.dsdp is not installed"
        dsdp.options = options
        if p: raise ValueError, "sdp() with the 'solver=dsdp' option "\
            "does not handle problems with equality constraints"
        dsdpstatus, x, r, zl, zs = dsdp.sdp(c, Gl, hl, Gs, hs)
        sl = hl - Gl*x
        ss = [ hs[k] - matrix(Gs[k]*x, hs[k].size) for k in 
            xrange(len(hs)) ]
        if dsdpstatus == 'DSDP_PDFEASIBLE':
            y = matrix(0.0, (0,1))
            status = 'optimal'
        elif dsdpstatus == 'DSDP_UNBOUNDED':
            pcost = blas.dot(c,x)
            x /= -pcost
            sl /= -pcost
            ss = [sk / -pcost for sk in ss]
            zl, zs = None, None
            status = 'dual infeasible'
        elif dsdpstatus == 'DSDP_INFEASIBLE':
            dcost = -blas.dot(hl,zl) - misc.sdot(hs, zs)
            zl /= -dcost 
            zs = [zs / -docst for zk in zs]
            y = matrix(0.0, (0,1))
            x, sl, ss = None, None, None
            status = 'primal infeasible'
        else:
            status = 'unknown'
            x, sl, ss, y, zl, zs, = None, None, None, None, None, None

        return {'status': status, 'x': x, 'y': y, 'sl': sl, 'ss': ss, 
            'zl': zl, 'zs': zs}
         

    def Fi(x, y, alpha=1.0, beta=0.0, trans='N'):
        base.gemv(Gl, x, y, alpha=alpha, beta=beta, trans=trans)

    def Fs(x, y, alpha=1.0, beta=0.0, trans='N'): 
        if trans == 'N':
            for k in xrange(N):
                # y[k] := alpha*mat(Gs[k]*x) + beta*y[k]
                base.gemv(Gs[k], x, y[k], alpha=alpha, beta=beta) 

        else:
            # y[i] := alpha*sum_k tr(mat(Gs[k][:,i])*x[k]) + beta*y[i] 
            # for i=0,...,n-1.

            for k in xrange(N):
                # Scale diagonal of x[k] by 1/2 and set the upper 
                # triangular part to zero, so that the matrix product 
                # of Gs[k]' and vec(x[k]) is equal to the inner 
                # product of mat(Gs[k][:j]) with x[k].  Then make the 
                # matrix-vector product.
                blas.scal(0.5, x[k], inc=ms[k]+1)
                for j in xrange(ms[k]):
                    blas.scal(0.0, x[k], offset=j+ms[k]*(j+1), 
                        inc=ms[k])
                if k == 0:
                    base.gemv(Gs[k], x[k], y, alpha=2*alpha, beta=beta,
                        trans='T')
                else:
                    base.gemv(Gs[k], x[k], y, alpha=2*alpha, beta=1.0,
                        trans='T')
                blas.scal(2.0, x[k], inc=ms[k]+1)
        
    def Fe(x, y, alpha=1.0, beta=0.0, trans='N'):
        base.gemv(A, x, y, alpha=alpha, beta=beta, trans=trans)

    
    # kktsolver(di, rti) returns a function for solving 
    #
    #    [ 0   A'  Gl'          Gs()'         ] [x ]   [bx ]
    #    [ A   0   0            0             ] [y ] = [by ].
    #    [ Gl  0   -diag(d)^2   0             ] [zl]   [bzl]
    #    [ Gs  0   0            -r*r'*()*r*r' ] [zs]   [bzs]
    #
    # di = 1./d is a positive mx1 'd' matrix;  
    # rti = inv(r') is a list of square matrices.
    #
    # The scaled zl and zs are returned: d.*zl instead of zl, r'*zs*r  
    # instead of zs.

    global kktmethod
    try: kktmethod = options['kktmethod']
    except KeyError: kktmethod = 5
    else: 
        if type(kktmethod) is not int or kktmethod not in [1,4,5]:
            raise ValueError, "options['kktmethod'] must be 1, 4, or 5"

    if kktmethod == 1:

        # The matrix 
        # 
        #    [ 0     A'   Glsc'  Gssc()' ] 
        #    [ A     0    0      0       ] 
        #    [ Glsc  0   -I      0       ] 
        #    [ Gssc  0    0     -(1/2)*I ]
        #
        # is factored as LDL, with:
        #
        #     Glsc = diag(d)^{-1}*G = diag(di)*G 
        #     Gssc() = rti'*Gs()*rti in packed storage with diagonal 
        #              elements scaled by 1/sqrt(2). 
        
        pms = [ mk*(mk+1)/2 for mk in ms ] 
        K = matrix(0.0, (n+p+ml+sum(pms), n+p+ml+sum(pms)))
        ipiv = matrix(0, (K.size[0],1))
        Gssc = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
        u = matrix(0.0, (K.size[0],1))

        def kktsolver(di, rti):
            blas.scal(0.0, K)

            # 3,3-block of K is -I.
            K[ (K.size[0]+1)*(p+n) : (K.size[0]+1)*(p+n+ml): 
                K.size[0]+1 ] = -1.0

            # 4,4-block of K is -I/2.
            K[ (K.size[0]+1)*(p+n+ml) :: K.size[0]+1 ] = -0.5

            # 2,1-block of K is A.
            K[n:n+p,:n] = A  

            # 3,1-block of K is diag(di)*Gl.
            K[n+p:n+p+ml, :n] = Gl
            for k in xrange(ml):
                blas.scal(di[k], K, n=n, offset=n+p+k, inc=K.size[0])

            # 4,1-block of K is rti'*G()*rti in packed storage with 
            # diagonal elements scaled by 1/sqrt(2).
            for j in xrange(n):
                # Gssc = rti' * mat(Gs[:,j]) * rti.
                for k in xrange(N): Gssc[k][:] = Gs[k][:,j]
                misc.cngrnc(rti, Gssc, Gssc, trans='T')

                # copy Gssc in packed storage to column j of K
                ind = n+p+ml + j*K.size[0]
                for k in xrange(N):
                    blas.scal(1.0/math.sqrt(2), Gssc[k], inc=ms[k]+1)
                    for i in xrange(ms[k]):
                        blas.copy(Gssc[k], K, n=ms[k]-i, 
                            offsetx=(ms[k]+1)*i, offsety=ind)
                        ind += ms[k]-i

            lapack.sytrf(K, ipiv)

            def solve_kkt(x, y, zl, zs):

                # Solve
                #
                #   [0    A'  Glsc' Gssc()'] [x        ]   [bx         ]
                #   [A    0   0     0      ] [y        ] = [by         ]
                #   [Glsc 0  -I     0      ] [d.*zl    ]   [di.*zl     ]
                #   [Gssc 0   0    -(1/2)*I] [2*r'*zs*r]   [rti'*zs*rti]
                #
                # and return x, y, d.*zl, r'*zs*r.

		blas.copy(x, u)
		blas.copy(y, u, offsety=n)
                misc.had2(zl, di)
                blas.copy(zl, u, offsety=n+p)
                misc.cngrnc(rti, zs, zs, trans='T')
                ind = n+p+ml
                for k in xrange(N):
                    blas.scal(1.0/math.sqrt(2), zs[k], inc=ms[k]+1)
                    for i in xrange(ms[k]):
                        blas.copy(zs[k], u, n=ms[k]-i, 
                            offsetx=(ms[k]+1)*i, offsety=ind)
                        ind += ms[k]-i    
                lapack.sytrs(K, ipiv, u)
                blas.copy(u, x, n=n)
                blas.copy(u, y, offsetx=n, n=p)
                blas.copy(u, zl, offsetx=n+p, n=ml)
                ind = n+p+ml
                for k in xrange(N):
                    for i in xrange(ms[k]):
                        blas.copy(u, zs[k], offsetx=ind, 
                            offsety=i*(ms[k]+1), n=ms[k]-i)
                        zs[k][i,i] *= math.sqrt(2)
                        ind += ms[k]-i    
                misc.sscal(0.5, zs)

            return solve_kkt

    elif kktmethod == 2: 

        pass   # not implemented

    elif kktmethod == 3: 

        pass   # not implemented

    elif kktmethod == 4: 

        # Factor the matrices
        #
        #     S = Gl'*diag(di)^2*Gl + Gs'(rti*rti'*Gs(x)*rti*rti') 
        #         + A'*A
        #     K = A*S^{-1}*A'. 
        #
        # Solve for y, x:
        #
        #     K*y = A * S^{-1} * (bx + Gl'*diag(di)^2*bzl  
        #           + Gs'(rti*rti'*bzs*rti*rti') + A'*by) - by
        #     S*x = bx + Gl'*diag(di)^2*bzl + Gs'(rti*rti'*bzs*rti*rti')
        #           + A'*by - A'*y.
        #
        # Compute scaled zl, zs:
        #
        #    zl = di .* (Gl*x - bzl) 
        #    zs = rti' * (Gs(x) - bzs) * rti.

        S, K = matrix(0.0, (n,n)), matrix(0.0, (p,p))
        if type(Gl) is matrix: 
            Glsc = matrix(0.0, (ml,n)) 
        else:
            Glsc = spmatrix(0.0, Gl.I, Gl.J, (ml,n)) 
        T = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
        Gssc = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
        zsc = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]

        def kktsolver(di, rti):

            # S = Gl'*diag(di)^2*Gl
            base.gemm(spmatrix(di, range(ml), range(ml), tc='d'), Gl, 
                Glsc, partial=True)
            base.syrk(Glsc, S, trans='T')

            # T = rti*rti' as a nonsymmetric matrix
            for k in xrange(N):
                blas.gemm(rti[k], rti[k], T[k], transB='T')

            for j in xrange(n):
                # Gssc[k] = T[k] * mat(Gs[k][:,j]) * T[k]
                for k in xrange(N): Gssc[k][:] = Gs[k][:,j]
                misc.cngrnc(T, Gssc, Gssc, trans='T')

                # N matrix vector products to get lower triangular part 
                # of column j of S.
                for k in xrange(N):
                    blas.scal(0.5, Gssc[k], inc=ms[k]+1)
                    for i in xrange(ms[k]):
                        blas.scal(0.0, Gssc[k], offset=i+ms[k]*(i+1), 
                            inc=ms[k])
                    base.gemv(Gs[k], Gssc[k], S, trans='T', n=n-j, 
                        alpha=2.0, beta=1.0, offsetA=j*(ms[k]**2), 
                        offsety=j*(n+1))
                    
            # S += A'*A    
            base.syrk(A, S, trans='T', beta=1.0)

            # factor S = L*L'
            lapack.potrf(S) 

            # Asct := L^{-1}*A',  factor K = Asct'*Asct
            if type(A) is matrix:
                Asct = A.trans()
            else:
                Asct = matrix(A.trans())
            blas.trsm(S, Asct)
            blas.syrk(Asct, K, trans='T')
            lapack.potrf(K)

            def solve_kkt(x, y, zl, zs):

                # zl := di.*zl = di.*bzl 
                # zsc := T*zs*T = T*bzs*T
                misc.had2(zl, di)
                misc.cngrnc(T, zs, zsc, trans='T')

                # x := x + Glsc'*zl + Gs'(zsc) + A'*y
                #    = bx + Gl'*diag(di)^2*bzl) + Gs'(T*bzs*T) + A'*by
                base.gemv(Glsc, zl, x, trans='T', beta=1.0)
                Fs(zsc, x, alpha=1.0, beta=1.0, trans='T') 
                base.gemv(A, y, x, trans='T', beta=1.0)

                # x := L^{-1}*x 
                #    = L^{-1} * (bx + Gl'*diag(di)^2*bzl + Gs'(T*bzs*T)
                #      + A'*by)
                blas.trsv(S, x)

                # y := Asc*x - y
                #    = A*S^{-1}* (bx + Gl'*diag(di)^2.*bzl + 
                #      Gs'(T*mat(bzs)*T) + A'*by) - by 
                base.gemv(Asct, x, y, trans='T', beta=-1.0)

                # y := K^{-1} * y
                #    = K^{-1} * (A*S^{-1}* (bx + Gl'*diag(di)^2*bzl + 
                #      Gs'(T*mat(bzs)*T) + A'*by) - by) 
                lapack.potrs(K, y)

                # x := x - Asc'*y
                base.gemv(Asct, y, x, alpha=-1.0, beta=1.0)

                # x := L^{-T} * x
                #    = S^{-1} * (bx + Gl'*diag(di)^2*bzl + 
                #      Gs'*(T*bzs*T) + A'*by - A'*y) 
                blas.trsv(S, x, trans='T')

                # zl := Glsc*x - zl 
                #     = di .* (Gl*x - bzl)
                base.gemv(Glsc, x, zl, beta=-1.0)

                # zs = rti' * (Gs(x) - bzs) * rti
                Fs(x, zs, alpha=1.0, beta=-1.0)
                misc.cngrnc(rti, zs, zs, trans='T')

            return solve_kkt 

    elif kktmethod == 5:  

        # Solve by eliminating the equality constraints and using 
        # QR factorizations
        #
        #     A' = Q1*R1,   G*Q1 = Q3*R3
        # 
        # where G = [Glsc; Gssc], Glsc = diag(di)*G, and 
        # Gssc = rti'*Gs()*rti (in packed storage with off-diagonal 
        # elements scaled by sqrt(2)).

        # A' = [Q1, Q2] * [R1; 0]
        if type(A) is matrix:
            QA = +A.trans()
        else:
            QA = matrix(A.trans())
        tauA, tauG = matrix(0.0, (p,1)), matrix(0.0, (n-p,1))
        lapack.geqrf(QA, tauA)

        pms = [ mk*(mk+1)/2 for mk in ms ] 
        Gssc = [ matrix(0.0, (m_k,m_k)) for m_k in ms ]
        G = matrix(0.0, (ml+sum(pms), n))
        z = matrix(0.0, (ml+sum(pms),1)) 
        w = matrix(0.0, (ml+sum(pms),1))
        v = matrix(0.0, (n,1))
        def kktsolver(di, rti):

            # G = [Glsc; Gssc] where , Glsc = diag(di)*G, and 
            # Gssc = rti'*Gs()*rti (in packed storage with off-diagonal 
            # elements scaled by sqrt(2)).

            G[:ml,:] = Gl
            for k in xrange(ml):
                blas.scal(di[k], G, n=n, offset=k, inc=G.size[0])

            for j in xrange(n):
                # Gssc = sqrt(2) * rti' * mat(Gs[:,j]) * rti.
                for k in xrange(N): Gssc[k][:] = Gs[k][:,j]
                misc.cngrnc(rti, Gssc, Gssc, trans='T', 
                    alpha=math.sqrt(2))
                # copy Gssc in packed storage to column j of G and 
                # scale diagonal elements by 1/sqrt(2).
                ind = ml + j*G.size[0]
                for k in xrange(N):
                    blas.scal(1.0/math.sqrt(2), Gssc[k], inc=ms[k]+1)
                    for i in xrange(ms[k]):
                        blas.copy(Gssc[k], G, n=ms[k]-i, 
                            offsetx=(ms[k]+1)*i, offsety=ind)
                        ind += ms[k]-i

            # G = [G1, G2] := G * [Q1, Q2]
            lapack.ormqr(QA, tauA, G, side='R')

            # QR factorization G2 := [Q3, Q4] * [R3; 0]. 
            lapack.geqrf(G, tauG, n=n-p, offsetA=G.size[0]*p)

            def solve_kkt(x, y, zl, zs):

                # Solve
                #
                #     [0  A'  G'] [x ]   [bx]
                #     [A  0   0 ] [y ] = [by]
                #     [G  0  -I ] [z ]   [bz]
                #
                # where
                #    z = [d.*zl; r'*zs*r] (with r'*zs*r in packed 
                #        storage with off-diagonal elements scaled by 
                #        sqrt(2))
                #    bz = [di.*bzl; rti'*bzs*rti] (with rti'*bzs*rti in
                #        packed storage with off-diagonal elements 
                #        scaled by sqrt(2))
                #
                # If we eliminate x = [Q1 Q2] * [ R1^{-T}*by; v ] the 
                # equations reduce to
                # 
                #        G2'*z = Q2'*bx
                #     G2*v - z = bz - G1*R1^{-T}*by
                #        R1*y  = Q1'*bx - G1'*z.
                #
                # This can be solved in 5 steps:
                #
                #     w := bz - G1*R1^{-T}*by 
                #     u := Q3'*w + R3^{-T}*Q2'*bx 
                #     z := Q3*u - w
                #     x := [Q1, Q2] * [ R1^{-T}*by;  R3^{-1}*u]
                #     y := R1^{-1} * (Q1'*bx - G1'*z).

                # w := bz = [bzlc; bzsc]
                misc.had2(zl, di)
                blas.copy(zl, w)
                misc.cngrnc(rti, zs, zs, trans='T', alpha=math.sqrt(2))
                ind = ml
                for k in xrange(N):
                    blas.scal(1.0/math.sqrt(2), zs[k], inc=ms[k]+1)
                    for i in xrange(ms[k]):
                        blas.copy(zs[k], w, n=ms[k]-i, 
                            offsetx=(ms[k]+1)*i, offsety=ind)
                        ind += ms[k]-i    

                # v := [Q1'*bx;  R3^{-T}*Q2'*bx]
                blas.copy(x, v)
                lapack.ormqr(QA, tauA, v, trans='T') 
                lapack.trtrs(G, v, uplo='U', trans='T', n=n-p,
                    offsetA=G.size[0]*p, offsetB=p)

                # x[:p] := R1^{-T}*by 
                blas.copy(y, x)
                lapack.trtrs(QA, x, uplo='U', trans='T', n=p)

                # w := w - G1*x[:p] 
                #    = bz - G1*R1^{-T}*by 
                blas.gemv(G, x, w, alpha=-1.0, beta=1.0, n=p)

                # z := [Q3'*w + v[p:]; 0]
                #    = [Q3'*w + R3^{-T}*Q2'*bx; 0]
                blas.copy(w, z)
                lapack.ormqr(G, tauG, z, trans='T', k=n-p, 
                    offsetA=G.size[0]*p)
                blas.axpy(v, z, offsetx=p, n=n-p)
                blas.scal(0.0, z, offset=n-p)

                # x[p:] := R3^{-1}*z[:n-p]  
                blas.copy(z, x, offsety=p, n=n-p)
                lapack.trtrs(G, x, uplo='U', n=n-p, offsetA=G.size[0]*p,
                    offsetB=p)

                # x is now [ R1^{-T}*by;  Q3'*w + R3^{-T}*Q2'*bx ]
                # x := [Q1 Q2]*x
                lapack.ormqr(QA, tauA, x) 
 
                # z := [Q3, Q4] * z - w
                lapack.ormqr(G, tauG, z, k=n-p, offsetA=G.size[0]*p)
                blas.axpy(w, z, alpha=-1.0)

                # y := R1^{-1} * ( v[:p] - G1'*z )
                #    = R1^{-1} * ( Q1'*bx - G1'*z )
                blas.copy(v, y, n=p)
                blas.gemv(G, z, y, n=p, trans='T', alpha=-1.0, beta=1.0)
                lapack.trtrs(QA, y, uplo='U', n=p) 

                # zl = z[:ml]
                blas.copy(z, zl, n=ml)

                # zs = z[ml:] unpacked and with offdiagonal elts 
                # scaled by 1/sqrt(2)
                ind = ml
                for k in xrange(N):
                     for i in xrange(ms[k]):
                         blas.copy(z, zs[k], offsetx=ind, 
                             offsety=i*(ms[k]+1), n=ms[k]-i)
                         ind += ms[k]-i    
                     blas.scal(math.sqrt(2.0), zs[k], inc=ms[k]+1)
                misc.sscal(1.0/math.sqrt(2.0), zs)

            return solve_kkt

    return conelp(c, kktsolver, Gl=Fi, hl=hl, Gs=Fs, hs=hs, A=Fe, b=b, 
        primalstart=primalstart, dualstart=dualstart)
